/*
 * mcc.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: freescale
 */

#include "mcc.h"

#include <stdio.h>
#include <string.h>

//#include <linux/mcc_config.h>
//#include <linux/mcc_common.h>
//#include <mcc_api.h>
extern "C"
{
#include <linux/mcc_config.h>
#include <linux/mcc_common.h>
#include <mcc_api.h>
}

// MCC constants
#define M4_CORE 1
#define MQX_NODE 0
#define MQX_PORT 1

//used by accelerometer-cgi from webgl calling
//#define THIS_NODE 0
#define THIS_NODE 1
#define THIS_PORT 1

static MCC_ENDPOINT m4ep =
{ M4_CORE, MQX_NODE, MQX_PORT };
static MCC_ENDPOINT a5ep; // filled in when created

/*
struct mcc_msg_struct {
    MCC_ENDPOINT ep_to_send_data;
    int cmd;
    int status;
    int sequence;
    int x;
    int y;
    int z;
} packed;
*/
struct mcc_msg_struct {
    MCC_ENDPOINT ep_to_send_data;
    int cmd;
    int status;
    int sequence;
    int x;
    int y;
    int z;
    int pot;
} packed;


int ret_value, rec_len;
MCC_INFO_STRUCT mcc_info;
struct mcc_msg_struct msg;

int CreateMcc(void)
{
    // initialize the sub-system
    ret_value = mcc_initialize(THIS_NODE);
    if (ret_value != MCC_SUCCESS)
    {
        printf("CA5-OOBE:Error: mcc_initialize error: %d for node: %d\n", ret_value,
                THIS_NODE);
        return ret_value;
    }

    // verify compatible versions
    ret_value = mcc_get_info(THIS_NODE, &mcc_info);
    if (ret_value != MCC_SUCCESS)
    {
        printf("CA5-OOBE:Error: mcc_get_info error: %d for node: %d\n", ret_value,
                THIS_NODE);
        return ret_value;
    }

    if (strcmp(mcc_info.version_string, MCC_VERSION_STRING) != 0)
    {
        printf("CA5-OOBE:Error: different versions of the MCC library on each core!\n");
        mcc_destroy(THIS_NODE);
        return MCC_ERR_DEV;
    }

    // create this side's mailbox
    ret_value = mcc_create_endpoint(&a5ep, THIS_PORT);
    if (ret_value != MCC_SUCCESS)
    {
        printf("CA5-OOBE:Error: mcc_create_endpoint error: %d for node,port: %d, %d\n",
                ret_value, THIS_NODE, THIS_PORT);
        mcc_destroy(THIS_NODE);
        return ret_value;
    }

    printf("CA5-OOBE: Endpoint is %d,%d,%d \n", a5ep.core, a5ep.node, a5ep.port);

    return 0;
}


int GetMccData(int *x, int *y, int *z, int *trim)
{

    // cgi response needs to start with this
    //printf("Content-type: text/html\n\n");

    msg.ep_to_send_data=a5ep;  //filled in init
    msg.cmd=0x11;
    msg.status=0x22;
    msg.sequence=0x33;
    msg.x=0x44;
    msg.y=0x55;
    msg.z=0x66;
    msg.pot=0x77;


    printf("CA5-OOBE: Sending request to %d,%d,%d from %d,%d,%d\n", m4ep.core, m4ep.node, m4ep.port,a5ep.core, a5ep.node, a5ep.port);
    ret_value = mcc_send(&m4ep, &msg, sizeof(msg), 500000);
    if (MCC_SUCCESS != ret_value)
    {
        printf("CA5-OOBE:Error: mcc_send error: %d\n", ret_value);
        mcc_destroy(THIS_NODE);
        return 1;
    }

    printf("CA5-OOBE: Receiving request from %d,%d,%d\n", a5ep.core, a5ep.node, a5ep.port);
    ret_value = mcc_recv_copy(&a5ep, (char *) &msg, sizeof(msg),
            (MCC_MEM_SIZE *) &rec_len, 0xFFFFFFFF);
    if (MCC_SUCCESS != ret_value)
    {
        printf("CA5-OOBE:Error: mcc_recv_copy error: %d\n", ret_value);
        mcc_destroy(THIS_NODE);
        return 1;
    }

    *x = msg.x;
    *y = msg.y;
    *z = msg.z;
    *trim = msg.pot;

    printf("CA5-OOBE:Accelerometer data received: %d,%d,%d,%d\n", msg.x, msg.y, msg.z,msg.pot);
    return 0;

}

int DestroyMcc(void)
{
    printf("CA5-OOBE:Destroing MCC endpoint\n");
    mcc_destroy_endpoint(&a5ep);
    mcc_destroy(THIS_NODE);

    return 0;
}
