/*
 * oobe.cpp
 *
 *  Created on: 09.04.2013
 *      Author: Jiri, Freescale
 */

#include "oobe.h"
#include "mcc.h"
#include <QtGui>
#include <QNetworkInterface>

//----------------------------------------------------------------------------
oobe::oobe(QWidget *parent) :
        QWidget(parent)
{
    const int MAX_LENGTH = 200;
    char config_buffer[MAX_LENGTH];
    printf("CA5-OOBE: Start of constructor\n");

    // Layout - set up user interface and hide the splash image
    ui.setupUi(this);
    ui.textBrowser->hide();
    //ui.lcdNumber_trim->hide();
    ui.label_info_text->setText("Welcome to the Vybrid world");
    splash_label = new QLabel(this);
    splash_pixmap = QPixmap(QString::fromUtf8(":/freescale.jpg"));
    splash_label->setPixmap(splash_pixmap);
    splash_label->resize(splash_pixmap.width(), splash_pixmap.height());
    splash_label->move(10, 10);
    //splash_label->move((screen.width() - splash_label->width())/2,(screen.height() - splash_label->height() - SPACE));

    //Configure - load pipeline configurations
    load_config_file(ui.fileCombo);
    if (get_config("pipeline", config_buffer) == false)
        puts("Error : no pipeline\n");

    // Initialize timer with 1000ms callback
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(TimerCallback()));
    timer->start(100);

    // Gstreamer initialization
    gst_player = new GstPlayer();
    gst_bus = NULL;

    // Default: is not playing
    is_playing = false;

    // Gstreamer overrides alpha blend, so this sets it back to desired blend
    set_alpha(255);

    // Reset accelerometer and analog data
    m_accel_x = 0;
    m_accel_y = 0;
    m_accel_z = 0;
    m_trim = 0;
    m_runtime = 0;

    // LEDs init - open the device
    // GPIO init -> pin is Off -> TWR-VF600 LED is on
    led1.Init(g_cgpio22);
    led2.Init(g_cgpio23);
    led3.Init(g_cgpio24);
    led4.Init(g_cgpio25);

    // LEDs blink invitation
    for (int i = 0; i < 2; i++)
    {
        led1.LedOn();
        led2.LedOn();
        led3.LedOn();
        led4.LedOn();
        usleep(50000);
        led1.LedOff();
        led2.LedOff();
        led3.LedOff();
        led4.LedOff();
        usleep(50000);
    }
    CreateMcc();
    printf("CA5-OOBE: End of constructor\n");
}

//----------------------------------------------------------------------------
oobe::~oobe()
{
}

//----------------------------------------------------------------------------
// Return true if EOS; else, returns false
bool oobe::CheckBus()
{
    GstMessage *msg;
    bool eos_reached = false;

    // If this is a NULL pointer, just return
    if (gst_bus == NULL
    )
        return eos_reached;

    while ((msg = gst_bus_pop(gst_bus)) != NULL)
    {
        //g_print("\nBus got message!");
        switch (GST_MESSAGE_TYPE(msg))
        {
        case GST_MESSAGE_EOS:
        {
            g_print("\nEOS reached. Now stopping pipeline");
            eos_reached = true;
            break;
        }
        case GST_MESSAGE_ERROR:
        {
            g_print("\nGstreamer error");
            GError *error;
            gchar *debug;
            gst_message_parse_error(msg, &error, &debug);
            g_print("\nError: %s", error->message);
            g_error(debug);
            g_error_free(error);
            break;
        }
        default:
            break;
        }
        gst_message_unref(msg);
    }
    return eos_reached;
}

//----------------------------------------------------------------------------
void oobe::TimerCallback()
{
    m_runtime++;
    GetMccData(&m_accel_x, &m_accel_y, &m_accel_z, &m_trim);
    if (m_accel_x > 100)
        m_accel_x = 100;
    else if (m_accel_x < -100)
        m_accel_x = -100;

    if (m_accel_y > 100)
        m_accel_y = 100;
    else if (m_accel_y < -100)
        m_accel_y = -100;

    if (m_accel_z > 100)
        m_accel_z = 100;
    else if (m_accel_z < -100)
        m_accel_z = -100;

    if (m_trim > 100)
        m_trim = 100;
    else if (m_trim < 0)
        m_trim = 0;

    ui.progressBar_x->setValue(m_accel_x);
    ui.progressBar_y->setValue(m_accel_y);
    ui.progressBar_z->setValue(m_accel_z);

    if (m_runtime == 300)
    {
        m_runtime = 0;
    }

    if (m_runtime == 50)
    {
        ui.textBrowser->hide();
        ui.label_info_text->setText("Toggle LEDs by buttons");
    }
    else if (m_runtime == 100)
    {
        ui.label_info_text->setText("Play the video from the list");
    }
    else if (m_runtime == 150)
    {
        ui.label_info_text->setText("Tilt the module - see LEDs / screen");
    }
    else if (m_runtime == 200)
    {
        ui.label_info_text->setText("Click on WWW button for WebGL demo");
    }
    else if (m_runtime == 250)
    {
        ui.label_info_text->setText("");
    }

    ui.dial_trim->setValue(m_trim);

    if ((m_runtime % 10) > 4)
        ui.radioButton_4->setChecked(false);
    else
        ui.radioButton_4->setChecked(true);


    ui.lcdNumber_trim->display(m_runtime % 10);

    if ((m_accel_x > 14 || m_accel_x < -14))
    {
        m_mode = MODE_TILT;
    }

    //If in TILT mode, turn on/off LED's based on how far tilted
    if (m_mode == MODE_TILT)
    {
        //If level again, go back to TOUCH mode
        if (m_accel_x <= 14 && m_accel_x >= -14)
        {
            m_mode = MODE_TOUCH;
            //Set all outputs to on
            led1.LedOn();
            led2.LedOn();
            led3.LedOn();
            led4.LedOn();
        }
        else
        {
        }
    }

    /* Turn on LED's based on how far tilted */
    if (m_accel_x < -52)
    {
        TiltOutput(0x0);
    }
    else if (m_accel_x < -40)
    {
        TiltOutput(0x1);
    }
    else if (m_accel_x < -28)
    {
        TiltOutput(0x3);
    }
    else if (m_accel_x < -14)
    {
        TiltOutput(0x7);
    }
    else if (m_accel_x > 52)
    {
        TiltOutput(0x0);
    }
    else if (m_accel_x > 40)
    {
        TiltOutput(0x8);
    }
    else if (m_accel_x > 28)
    {
        TiltOutput(0xC);
    }
    else if (m_accel_x > 14)
    {
        TiltOutput(0xE);
    }

    if (CheckBus())
    {
        Stop();
    }

}

//----------------------------------------------------------------------------
void oobe::TiltOutput(char signal)
{
    signal &= 0xF;

    if (signal & 0x8)
    {
        led1.LedOn();
    }
    else
    {
        led1.LedOff();
    }

    if (signal & 0x4)
    {
        led3.LedOn();
    }
    else
    {
        led3.LedOff();
    }

    if (signal & 0x2)
    {
        led2.LedOn();
    }
    else
    {
        led2.LedOff();
    }

    if (signal & 0x1)
    {
        led4.LedOn();
    }
    else
    {
        led4.LedOff();
    }
}

//----------------------------------------------------------------------------
void oobe::Clicked1()
{
    if (led1.m_ivalue == 0)
        led1.On();
    else
        led1.Off();
    /*
     m_accel_x += 10;
     if (m_accel_x > 100)
     {
     m_accel_x = 0;
     }
     */

    puts("LED1 is toggled");
    m_runtime = 0;
    ui.label_info_text->setText("LED1 is toggled");
}

//----------------------------------------------------------------------------
void oobe::Clicked2()
{
    if (led2.m_ivalue == 0)
        led2.On();
    else
        led2.Off();
    /*
     m_accel_y += 10;
     if (m_accel_y > 100)
     {
     m_accel_y = 0;
     }
     */

    puts("LED2 is toggled");
    m_runtime = 0;
    ui.label_info_text->setText("LED2 is toggled");

}
//----------------------------------------------------------------------------
void oobe::Clicked3()
{
    if (led3.m_ivalue == 0)
        led3.On();
    else
        led3.Off();
    /*
     m_accel_z += 10;
     if (m_accel_z > 100)
     {
     m_accel_z = 0;
     }
     */

    puts("LED3 is toggled");
    m_runtime = 0;
    ui.label_info_text->setText("LED3 is toggled");

}

//----------------------------------------------------------------------------
void oobe::Clicked4()
{
    if (led4.m_ivalue == 0)
        led4.On();
    else
        led4.Off();
    /*
     m_trim += 10;
     if (m_trim > 100)
     {
     m_trim = 0;
     }
     */

    ui.dial_trim->setValue(m_trim);

    puts("LED4 is toggled");
    m_runtime = 0;
    ui.label_info_text->setText("LED4 is toggled");

}

//----------------------------------------------------------------------------
void oobe::Play()
{
    if (is_playing == false)
    {
        // Play stream
        m_runtime = 0;
        ui.label_info_text->setText("Playing the video...");
        puts("Playing the video...");

        //blank_vybrid("/dev/fb0"); //QT
        blank_vybrid("/dev/fb2"); //video

        gst_player->set_mode(VIDEO);
        gst_bus = gst_player->get_bus();

        gst_player->play();
        is_playing = true; // video is_playing

        ui.pushButton_dummy->setGeometry(5, 5, 465, 262); // whole screen almost

        // If beginning to play the first time, we have to pause to let
        // GStreamer get its ducks in a row
        usleep(3500000);

        set_alpha(0); //hide qt
    }

}

//----------------------------------------------------------------------------
void oobe::Stop()
{
    puts("Stopping the video...");
    ui.pushButton_dummy->setGeometry(20, 30, 100, 20); // back to small

    //blank_vybrid("/dev/fb0"); //QT
    blank_vybrid("/dev/fb2"); //video

    //reset player
    gst_player->stop();
    gst_bus = gst_player->get_bus();

    is_playing = false;
    usleep(500000);

    set_alpha(255); //QT visible
    m_runtime = 0;
    ui.label_info_text->setText("Video is stopped...");

    /*
     puts("Dimming the video...\n");
     if (is_playing)
     {
     ui.pushButton_dummy->setGeometry(20, 60, 151, 21); // back to small
     set_alpha(127); // QT partly visible
     gst_player->pause();
     }
     */

}

//----------------------------------------------------------------------------
void oobe::Web()
{
    QString displayIpadress = "Exit; type ";
    QString ipadress;
    QString displayIpadressEnd = " in web browser";
    //QNetworkInterface iface = QNetworkInterface::interfaceFromName("eth0");
    //ipadress = iface.addressEntries()[0].ip().toString();
    QNetworkInterface iface = QNetworkInterface::interfaceFromName("eth0");
    QString ip = "0.0.0.0";
    if (iface.isValid())
        ipadress = iface.addressEntries()[0].ip().toString();

    displayIpadress.append(ipadress);
    displayIpadress.append(displayIpadressEnd);
    m_runtime = 0;
    ui.label_info_text->setText(displayIpadress);
    ui.textBrowser->setGeometry(QRect(5, 51, 470, 218));
    ui.textBrowser->show();
    //ui.label_info_text->setText("Type 192.168.1.3 in your web browser");
}

//----------------------------------------------------------------------------
// Dereference pipeline, end program
void oobe::End()
{
    m_runtime = 0;
    puts("Exiting OOBE demo...");

    gst_player->destroy();

    delete (timer);
    delete (gst_player);

    DestroyMcc();

    m_runtime = 0;
    puts("Thank you for interesting in Vybrid product.");

    blank_vybrid("/dev/fb0"); //QT
    exit(0);
    led1.LedOff();

}

/*
 * Used by Accel_Task, this function turns on
 *  the LED's specified
 *
 * Each bit represents an LED to turn on
 *
 */
